/*****************************************************************************
**                                                                          **
**              PHOENIX CONTACT GmbH & Co., D-32819 Blomberg                **
**              (c) Phoenix Contact 04/1998                                 **
**                                                                          **
******************************************************************************
**                                                                          **
** Project      : HIGH LEVEL LANGUAGE INTERFACE                             **
**                for PC-based INTERBUS G4 standard controllers (SC series) **
** Sub-Project  : Application Interface for 'C/C++'                         **
**                Master Header                                             **
*****************************************************************************/

/*
** File         : IBSG4HLI.H
** Component(s) : -
** Device(s)    : independent
** Terminology  : -
**              :
** Definition   : S. Gallmann
** Author       : S. Gallmann
**              :
** Version      : 1.04
** Status       : Released
**              :
** Description  : Master header for 'C/C++' applications
**              :
******************************************************************************
** History      :
**
** Date   Version  Build Author Description
** ---------------------------------------------------------------------------
** 05/98  1.01      01    SG    Released
** ---------------------------------------------------------------------------
** 06/98  1.02      02    SG    Fixes and changes see README documentation
** 09/98  1.03      03    SG    Fixes and changes see README documentation
** 11/98  1.04      01    SG    Fixes and changes see README documentation
*/


/*--------------------------------------------------------------------*/
/*                    I.  DEFINE STATEMENTS                           */
/*--------------------------------------------------------------------*/

#ifndef IBS_G4_HLI_H


/* ****************************************************************
             P L A T F O R M  D E P A N D A N C Y
***************************************************************** */


#ifdef HLI_DOS_VERSION
       #define HLI_OS_DEFINED
       #define HLI_OS_DOS
       #define HLI_NODE_0
       #define HLI_PCP_FILE_SUPPORT
#endif

#ifdef HLI_WIN16_VERSION
       #define HLI_OS_DEFINED
       #define HLI_OS_WIN16
       #define HLI_NODE_0
       #define HLI_PCP_FILE_SUPPORT
#endif

#ifdef HLI_WIN32_VERSION
       #define HLI_OS_DEFINED
       #define HLI_OS_WIN32
       #define HLI_NODE_0
       #define HLI_ETH_SUPPORT
       #define HLI_COPCOM_SUPPORT
       #define HLI_PCP_FILE_SUPPORT
       #define HLI_PCCARD_SUPPORT
       #define HLI_PCI_SUPPORT 
       #define HLI_PCP_RW_NAME_SUPPORT
#endif

#ifdef HLI_COP486_DOS_VERSION
       #define HLI_OS_DEFINED
       #define HLI_OS_DOS
       #define HLI_NODE_2
       #define HLI_PCP_FILE_SUPPORT
#endif

#ifdef HLI_COP486_VXWORKS_VERSION
       #define HLI_OS_DEFINED
       #define HLI_OS_VXWORKS
       #define HLI_NODE_2
       #define HLI_PCP_FILE_SUPPORT
#endif

#ifdef HLI_WINCE_VERSION
       #define HLI_OS_DEFINED
       #define HLI_OS_WINCE
       #define HLI_NODE_0
       #define HLI_PCP_RW_NAME_SUPPORT
#endif


/* ############### Check target definition ############# */


#ifndef HLI_OS_DEFINED

 #error NO TARGET OPERATING SYSTEM DEFINED !

#endif


#define IBS_G4_HLI_H


/* ############## MS-DOS-compatible systems ############ */

#ifdef HLI_OS_DOS

 #define VOID                           void
 #define INT8                           signed char
 #define INT16                          signed short
 #define INT32                          signed long
 #define FLOAT                          float
 #define CHAR                           char
 #define USIGN8                         unsigned char
 #define USIGN16                        unsigned short
 #define USIGN32                        unsigned long
 #define HLIBOOL                        unsigned short
 #define HLIRET                         unsigned short

 /* functions declaration macros */

 #define IBSHLI                         extern USIGN16 _far _pascal
 #define REF                            _far *
 #define VOIDREF                        void _far *
 #define STRINGREF                      CHAR _far *

#endif




/* ############ Windows 3.x, Win 95-16-Bit ############# */

#ifdef HLI_OS_WIN16

 #include <WINDOWS.H>                   /* HLI needs the WIN16 API definitions */

 #define VOID                           void
 #define INT8                           signed char
 #define INT16                          signed short
 #define INT32                          signed long
 #define CHAR                           char
 #define FLOAT                          float
 #define USIGN8                         unsigned char
 #define USIGN16                        unsigned short
 #define USIGN32                        unsigned long
 #define HLIBOOL                        unsigned short
 #define HLIRET                         unsigned short

/* functions declaration macros */

 #define IBSHLI                         extern USIGN16 WINAPI
 #define REF                            FAR *
 #define VOIDREF                        LPVOID
 #define STRINGREF                      LPSTR

#endif




/* ########### Windows 95-32-Bit, Windows NT ############ */

#ifdef HLI_OS_WIN32

 #include <WINDOWS.H>                   /* HLI needs the WIN32 API definitions */

 #define VOID                           void
 #define INT8                           signed char
 #define INT16                          signed short
 #define INT32                          signed long
 #define USIGN8                         unsigned char
 #define USIGN16                        unsigned short
 #define USIGN32                        unsigned long
 #define HLIBOOL                        unsigned short
 #define HLIRET                         unsigned short

 /* functions declaration macros */

 #ifdef  _MSC_VER
  #define IBSHLI                        extern __declspec(dllexport) USIGN16 WINAPI
 #else
  #define IBSHLI                        extern USIGN16 WINAPI
 #endif
 #define IBSHLICALLBACK                 VOID WINAPI
 #define REF                            *
 #define VOIDREF                        LPVOID
 #define STRINGREF                      LPSTR

#endif


/* ################# Windows CE 2.0 Version ################## */

#ifdef HLI_OS_WINCE

 #include <WINDOWS.H>                   /* HLI needs the WIN CE API definitions */

 #define VOID                           void
 #define INT8                           signed char
 #define INT16                          signed short
 #define INT32                          signed long
 #define CHAR                           wchar_t
 #define USIGN8                         unsigned char
 #define USIGN16                        unsigned short
 #define USIGN32                        unsigned long
 #define HLIBOOL                        unsigned short
 #define HLIRET                         unsigned short

 /* functions declaration macros */

 #ifdef  _MSC_VER
  #define IBSHLI                        extern __declspec(dllexport) USIGN16 WINAPI
 #else
  #define IBSHLI                        extern USIGN16 WINAPI
 #endif
 #define IBSHLICALLBACK                 VOID WINAPI
 #define REF                            *
 #define VOIDREF                        LPVOID
 #define STRINGREF                      CHAR *

#endif


/* ################## WindRiver's VxWorks ############## */

#ifdef HLI_OS_VXWORKS

 #include <VxWorks.h>                   /* HLI needs the VxWorks definitions */

 #define CHAR                           char
 #define FLOAT                          float
 #define USIGN8                         unsigned char
 #define USIGN16                        unsigned short
 #define USIGN32                        unsigned long
 #define HLIBOOL                        unsigned short
 #define HLIRET                         unsigned short

 /* functions declaration macros */

 #define IBSHLI                         extern USIGN16
 #define REF                            *
 #define VOIDREF                        void *
 #define STRINGREF                      char *
#endif



/* ****************************************************************
                  FUNCTIONAL CONSTANT DEFINITIONS
***************************************************************** */


/* general constants */

#ifndef TRUE
 #define TRUE                           1
#endif
#ifndef FALSE
 #define FALSE                          0
#endif


/* INTERBUS Process data object types */

#define T_IBS_BOOL                     HLIBOOL
#define T_IBS_BITSTRING                USIGN16
#define T_IBS_BYTE                     USIGN8
#define T_IBS_WORD                     USIGN16
#define T_IBS_DWORD                    USIGN32


/* Controller board access constants ;
   use these constants for all functions that have the <CID> parameter */

 /* IBS PC ISA SC/I-T  Board 1 ... 4 */
#define ISASC1                         0x01
#define SCCOP                          0x01
#define ISASC2                         0x02
#define ISASC3                         0x03
#define ISASC4                         0x04

 /* IBS ETH DSC/I-T  Controller 1 ... 4 */
#define ETHDSC1                        0x11
#define ETHDSC2                        0x12
#define ETHDSC3                        0x13
#define ETHDSC4                        0x14

/* IBS PCCARD  Controller 1 ... 4  */
#define PCCARD1                        0x21
#define PCCARD2                        0x22
#define PCCARD3                        0x23
#define PCCARD4                        0x24


/* IBS PCI  Board 1 ... 4   */
#define PCISC1                         0x31
#define PCISC2                         0x32
#define PCISC3                         0x33
#define PCISC4                         0x34


/* INTERBUS operating modes */

/* default operating mode of Controller (free running cycles): */
#define IBS_NORMAL                     0
#define IBS_STANDARD                   0

/* application controls INTERBUS-cycles : */
#define IBS_CONTROLLED                 1


/* classification of Process-data-object types used for
   PDO registration : */

#define IBS_PDO_INPUT                  0  /* input type */
#define IBS_PDO_OUTPUT                 1  /* output type */

#define IBS_PDO_BOOL                   1  /* bit object */
#define IBS_PDO_BITSTRING              2  /* bitstring object */
#define IBS_PDO_BYTE                   3  /* byte object */
#define IBS_PDO_WORD                   4  /* word object */
#define IBS_PDO_DWORD                  5  /* double word object */
#define IBS_SIG_BOOL                   6  /* bit object for signal path */
#define IBS_SIG_BITSTRING              7  /* bitstring object for signal path */
#define IBS_SIG_BYTE                   8  /* byte object for signal path */
#define IBS_SIG_WORD                   9  /* word object for signal path */
#define IBS_SIG_DWORD                  10 /* word object for signal path */


 /* Priority constants of PD-Thread */
#define LOW_PRIORITY                   0
#define NORMAL_PRIORITY                1
#define HIGH_PRIORITY                  2



/* ********* General error codes returned by HLI functions: ***** */

#define HLI_OK                         0x0
#define HLI_OKAY                       0x0

#define HLI_ACCESS_DENIED              0xFF01
#define HLI_INVALID_CID                0xFF02
#define HLI_NO_VALID_CFG               0xFF03
#define HLI_NO_LOG_WINDOW              0xFF04
#define HLI_LOGFILE_OPEN_ERROR         0xFF05
#define HLI_NO_MORE_EVENT              0xFF21

#define HLI_MSG_TOO_LONG               0xFF31 /* Only COP version relevant */
#define HLI_INVALID_HOST_DATA_ACCESS   0xFF32 /* Only COP version relevant */
#define HLI_NO_MESSAGE_AVAILABLE       0xFF33 /* Only COP version relevant */
#define HLI_SEND_MESSAGE_TIMEOUT       0xFF34 /* Only COP version relevant */

#define HLI_PCP_SET_NOT_FOUND          0xFF41
#define HLI_PCP_SET_INVALID            0xFF42


/* Constants of the <BusState> member of the <T_HLI_STATE> type  */

#define HLI_NOT_INITIALIZED            0x00   /* dto.            */

#define HLI_IBS_READY                  0x01   /* Controller is in "Ready" state */
#define HLI_IBS_ACTIVE                 0x02   /* Controller is in "Active" state*/
#define HLI_IBS_RUN                    0x03   /* Controller is in "Run" state   */

 /* Add. states for the application-controlled mode */
#define HLI_IBS_IDLE                   0x04   /* cycle finished */
#define HLI_IBS_PREPARING_CYCLE        0x05   /* processing I/O */
#define HLI_IBS_CAN_RUN                0x06   /* ready for next cycle*/

 /* Controller is in "Stop" state */
#define HLI_IBS_STOPPED                0xF001 /* Bus in controlled stop */
#define HLI_IBS_DETECT                 0xF002 /* Stop - controller is analysing the bus */
#define HLI_IBS_BUS_FAIL               0xF003 /* controller is in "bus fail" state */
#define HLI_IBS_CONTROLLER_FAIL        0xF004 /* Controller detected self-error */

/* constants for PCP device states */

#define HLI_PCP_DEVICE_READY           0x0
#define HLI_PCP_DEVICE_NOT_INITIATED   0xF012
#define HLI_PCP_DEVICE_BUSY            0xF013
#define HLI_PCP_DEVICE_NOT_READY       0xF014

/* <State> member constants for PCP service processing  */

#define HLI_PCP_SERVICE_OKAY           0x0
#define HLI_PCP_SERVICE_PENDING        0x0001

/* States for PCP device programs (<PRG_State> member) */

#define HLI_PCP_PRG_RESET              0x00
#define HLI_PCP_PRG_RUNNING            0x01
#define HLI_PCP_PRG_STOPPED            0x02

/* PCP object types used with PCP-object translation functions */

#define HLI_POT_BYTE                   0
#define HLI_POT_INT16                  1
#define HLI_POT_INT32                  2
#define HLI_POT_FLOAT                  3
#define HLI_POT_STRING                 4

/* PCP service types received from event parameters */

#define HLI_PCP_SERVICE_READ           1
#define HLI_PCP_SERVICE_WRITE          2
#define HLI_PCP_SERVICE_IDENTIFY       3
#define HLI_PCP_SERVICE_START          4
#define HLI_PCP_SERVICE_STOP           5
#define HLI_PCP_SERVICE_RESUME         6
#define HLI_PCP_SERVICE_RESET          7

/*depen */
#define HLI_PCP_SERVICE_WRITE_WITH_NAME     8
#define HLI_PCP_SERVICE_READ_WITH_NAME      9



/* ******************* HLI event type definitions ******************* */


/* HLI_EG_xxx constants define event groups to use with the
   IBS_HLI_EnableEvents(), IBS_HLI_DisableEvents(),
   IBS_HLI_EnableLogEvents() and IBS_HLI_DisableLogEvents() functions */

/* Event group: all possible events */
#define HLI_EG_ALL                     0x00FF

/* Event group: internal faults */

#define HLI_EG_INTERNAL                0x0001

#define HLI_EVENT_MEMORY_FAULT         0x0101
#define HLI_EVENT_OS_FAULT             0x0102
#define HLI_EVENT_NOTE_NO_PF_EV        0x0103
#define HLI_EVENT_NOTE_NO_PF_RESET_EV  0x0104

/* Event group: DDI errors          */

#define HLI_EG_DDI                     0x0002

#define HLI_EVENT_DDI_NOT_AVAILABLE    0x0201
#define HLI_EVENT_DDI_ERROR            0x0202
#define HLI_EVENT_NO_RIGHTS            0x0203
#define HLI_EVENT_FW_CNF_TIMEOUT       0x0204
#define HLI_EVENT_FW_CNF_ERROR         0x0205
#define HLI_EVENT_HOST_DDI_ERROR       0x0206

/* Event group: controller faults  */

#define HLI_EG_CTRL                    0x0004

#define HLI_EVENT_CONTROLLER_FAIL      0x0401
#define HLI_EVENT_SYSTEM_FAIL          0x0402
#define HLI_EVENT_WATCHDOG_ACTIVE      0x0403
#define HLI_EVENT_WATCHDOG_SHUT        0x0404

/* Event group: bus faults         */

#define HLI_EG_BUSFAULT                0x0008

#define HLI_EVENT_LOCATED_BUS_ERROR    0x0801
#define HLI_EVENT_UNLOCATED_BUS_ERROR  0x0802
#define HLI_EVENT_INVALID_DATA_CYCLE   0x0803

/* Event group: device state messages */

#define HLI_EG_DEVICEMSG               0x0010

#define HLI_EVENT_DEVICE_ERROR         0x1001
#define HLI_EVENT_DEVICE_MAU           0x1002
#define HLI_EVENT_DEVICE_RECONFIG      0x1003
#define HLI_EVENT_DEVICE_ERROR_RESET   0x1004
#define HLI_EVENT_DEVICE_STATE         0x1005

/* Event group: bus state changes  */

#define HLI_EG_BUSSTATE                0x0020

#define HLI_EVENT_BUS_START            0x2001
#define HLI_EVENT_BUS_STOP             0x2002
#define HLI_EVENT_ALARM_STOP           0x2003
#define HLI_EVENT_RB_SEGMENT_OFF       0x2004
#define HLI_EVENT_RB_SEGMENT_ON        0x2005
#define HLI_EVENT_LB_SEGMENT_OFF       0x2006
#define HLI_EVENT_LB_SEGMENT_ON        0x2007
#define HLI_EVENT_GROUP_OFF            0x2008
#define HLI_EVENT_GROUP_ON             0x2009
#define HLI_EVENT_BUS_WARNING          0x200A
#define HLI_EVENT_DEVICE_ENABLED       0x200B
#define HLI_EVENT_DEVICE_DISABLED      0x200C

/* Event group: PCP events         */

#define HLI_EG_PCP                     0x0040

#define HLI_EVENT_PCP_LOCAL_ABORT      0x4001
#define HLI_EVENT_PCP_REMOTE_ABORT     0x4002
#define HLI_EVENT_PCP_RECONNECTED      0x4003
#define HLI_EVENT_PCP_NOT_READY        0x4004

#define HLI_EVENT_PCP_SERVICE_FAILED   0x4005
#define HLI_EVENT_PCP_SERVICE_REJECTED 0x4006
#define HLI_EVENT_PCP_SERVICE_TIMEOUT  0x4007

/* Event group: application faults */

#define HLI_EG_APP                     0x0080
#define HLI_EVENT_NO_VALID_CFG         0x8001
#define HLI_EVENT_NULL_POINTER         0x8002
#define HLI_EVENT_INVALID_PARA         0x8003

#define HLI_EVENT_UNKNOWN_DEVNO        0x8004
#define HLI_EVENT_INVALID_IDCODE       0x8005
#define HLI_EVENT_NO_PCP_DEVICE        0x8006
#define HLI_EVENT_WRONG_NR_OF_DEVICES  0x8007
#define HLI_EVENT_UNKNOWN_GROUP        0x8008
#define HLI_EVENT_NO_INPUT_DEVICE      0x8009
#define HLI_EVENT_NO_OUTPUT_DEVICE     0x800A

#define HLI_EVENT_TOO_MANY_PDOS        0x800B
#define HLI_EVENT_INVALID_PDO_TYPE     0x800C
#define HLI_EVENT_INVALID_PDO_SIZE     0x800D
#define HLI_EVENT_INVALID_PDO_POINTER  0x800E
#define HLI_EVENT_INVALID_SYNCH_OP     0x800F
#define HLI_EVENT_IBS_NOT_RUNNING      0x8010
#define HLI_EVENT_IBS_DETECTING        0x8011

#define HLI_EVENT_DEVICE_NOT_INITIATED 0x8012
#define HLI_EVENT_DEVICE_BUSY          0x8013
#define HLI_EVENT_DEVICE_NOT_READY     0x8014

#define HLI_EVENT_CFG_INVALID_DEVNO    0x8021
#define HLI_EVENT_CFG_INVALID_IDCODE   0x8022
#define HLI_EVENT_CFG_INVALID_LEVEL    0x8023
#define HLI_EVENT_CFG_INVALID_GROUP    0x8024

#define HLI_EVENT_ACTION_NOT_ALLOWED   0x8031
#define HLI_EVENT_CANNOT_ACTIVATE_CFG  0x8032


/* **** Constants used by INTERBUS service functions ********** */

 /* IBS_HLI_Switchxxx() functions */
#define HLI_SEGMENT_OFF                0x0
#define HLI_SEGMENT_ON                 0x1
#define HLI_GROUP_OFF                  0x0
#define HLI_GROUP_ON                   0x1
#define HLI_DEVICE_DISABLED            0x0
#define HLI_DEVICE_ENABLED             0x1

 /* Constants for watchdog timeout values used by the
    IBS_HLI_ActivateWatchdog() function */
#define HLI_WDT_8                      0x00    /* Timeout = 8.2 ms    */
#define HLI_WDT_16                     0x04    /* Timeout = 16.4 ms   */
#define HLI_WDT_32                     0x08    /* Timeout = 32.8 ms   */
#define HLI_WDT_65                     0x0C    /* Timeout = 65.5 ms   */
#define HLI_WDT_131                    0x10    /* Timeout = 131.1 ms  */
#define HLI_WDT_262                    0x14    /* Timeout = 262.1 ms  */
#define HLI_WDT_524                    0x18    /* Timeout = 524.3 ms  */
#define HLI_WDT_1048                   0x1C    /* Timeout = 1048.6 ms */


/* *************** Event text & window constants **************** */

#define HLI_ENGLISH                    0x0     /* Language constant: English text */
#define HLI_DEUTSCH                    0x1     /* Language constant: German text  */

#define HLI_EW_FIXED                   0x0     /* User can not change filter setting */
#define HLI_EW_FILTERCHANGE            0x1     /* User can change filter setting */

/* HOST / COP Communication */

#define DATA_AREA_SIZE                 5120    /* Size of coupled data area */


/*--------------------------------------------------------------------*/
/*                   II.  TYPE DEFINITIONS                            */
/*--------------------------------------------------------------------*/


#ifdef __cplusplus
 extern "C" {            /* Assume C declarations for C++ */
#endif  /* __cplusplus */



/* struct to hold HLI state information */

typedef struct
        {
         USIGN16    BusState;
         USIGN16    DiagState;
         USIGN16    EventIndication;
        }
        T_HLI_STATE;

/* struct to hold date / time */

typedef struct
        {
         USIGN16    d;
         USIGN16    mo;
         USIGN16    y;
         USIGN16    h;
         USIGN16    m;
         USIGN16    s;
        }
        T_HLI_TIME;

/* struct to hold HLI Event */

typedef struct
        {
         T_HLI_TIME Timestamp;
         USIGN16    CID;
         USIGN16    Code;
         USIGN16    Param[5];
        }
        T_HLI_EVENT;

/* struct to hold Controller information */

typedef struct
        {
         USIGN8     reserved1;
         CHAR       Ident[32];
         USIGN8     reserved2;
         CHAR       FW_Version[8];
         USIGN8     reserved3;
         CHAR       HW_Version[4];
        }
        T_HLI_SC_INFO;

/* struct to hold Bus information */

typedef struct
        {
         USIGN16    Devices;
         USIGN16    IO_Points;
         USIGN16    PCP_Devices;
         USIGN16    PD_Length;
         USIGN32    Cycletime;
        }
        T_HLI_BUS_INFO;

/* struct to hold bus device data */

typedef struct
        {
         USIGN16    Segment;
         USIGN16    Position;
         USIGN16    ID_Code;
         USIGN16    LengthCode;
         USIGN16    Level;
         USIGN16    Group;
         USIGN16    Alternative;
         USIGN16    Deactivated;
         CHAR       Station[32];
        }
        T_HLI_DEVICE_DATA;

/* struct to hold bus device information */

typedef struct
        {
         USIGN16    Segment;
         USIGN16    Position;
         USIGN16    ID_Code;
         USIGN16    LengthCode;
         USIGN16    Level;
         USIGN16    Group;
         USIGN16    Alternative;
         USIGN16    InBitLength;
         USIGN16    OutBitLength;
         USIGN16    PCP_CR;
         USIGN16    Active;
        }
        T_HLI_DEVICE_INFO;

/* struct to hold PCP device information */

typedef struct
        {
         USIGN16    Segment;
         USIGN16    Position;
         USIGN16    State;
         USIGN16    ID_Code;
         USIGN16    LengthCode;
         USIGN8     reserved1;
         CHAR       Manufacturer[255];
         USIGN8     reserved2;
         CHAR       Name[255];
         USIGN8     reserved3;
         CHAR       Revision[255];
        }
        T_HLI_PCP_DEVICE_INFO;

/* struct to hold PCP-Data-Object */

typedef struct
        {
         USIGN16    Segment;
         USIGN16    Position;
         USIGN16    Index;
         USIGN16    Subindex;
         USIGN16    Length;
         USIGN16    State;
         USIGN8     Data[256];
        }
        T_HLI_PCP_RW;

/* Depen */
/* struct to hold PCP-Read_Write with Name Object */ 
typedef struct
        {
         USIGN16    Segment;
         USIGN16    Position;
         CHAR       VarName[1024];
         USIGN16    Length;
         USIGN16    State;
         USIGN8     Data[1024];
}T_HLI_PCP_RW_NAME; 



/* struct to hold PCP-Program-Object */

typedef struct
        {
         USIGN16    Segment;
         USIGN16    Position;
         USIGN16    Index;
         USIGN16    PRG_State;
         USIGN16    State;
        }
        T_HLI_PCP_PRG;

/* struct to hold diagnostic information */

typedef struct
        {
         USIGN32    Cycle_Count;
         USIGN32    Cycle_Error_Count;
         USIGN32    ID_Cycle_Count;
         USIGN32    ID_Cycle_Error_Count;
         USIGN32    Data_Cycle_Count;
         USIGN32    Data_Cycle_Error_Count;
        }
        T_HLI_SC_DIAG;


/* struct to hold message block */

typedef struct
        {
         USIGN16                  Code;
         USIGN16                  nParms;
         USIGN16                  Parms[512];
        }
        T_HLI_MSG;


#ifdef HLI_OS_WIN32
 typedef VOID ( WINAPI * LPHLICALLBACK)(VOID);
#endif


/*--------------------------------------------------------------------*/
/*                        H L I  FUNCTIONS                            */
/*--------------------------------------------------------------------*/


/* HLI version info */

IBSHLI  IBS_HLI_GetVersion(STRINGREF destination);

/* HLI-Init-functions */

/* Standard initialization: bus configuration based on 'Create_Configuration_Request' command */
IBSHLI  IBS_HLI_Init(USIGN16 CID, T_HLI_STATE REF lpIbsState, USIGN16 ibs_mode);
/* CFG initialization: bus configuration is parameterized by list */
IBSHLI  IBS_HLI_Init_CFG(USIGN16 CID, T_HLI_STATE REF lpIbsState, USIGN16 ibs_mode,
                         USIGN16 nDevices, T_HLI_DEVICE_DATA REF lpDeviceList);

IBSHLI  IBS_HLI_Exit(USIGN16 CID);


/* HLI-Event-Management */

IBSHLI  IBS_HLI_EnableEvents(USIGN16 CID, USIGN16 eventgroup);
IBSHLI  IBS_HLI_DisableEvents(USIGN16 CID, USIGN16 eventgroup);
IBSHLI  IBS_HLI_GetEventInfo(USIGN16 CID, T_HLI_EVENT REF Event);
IBSHLI  IBS_HLI_GetEventText(T_HLI_EVENT REF Event, USIGN16 Language, STRINGREF destination);

#if defined (HLI_OS_WIN16) || defined (HLI_OS_WIN32) || defined (HLI_OS_WINCE)
IBSHLI  IBS_HLI_CreateLogWindow(USIGN16 CID, USIGN16 Language, HLIBOOL change_filter);
IBSHLI  IBS_HLI_PopupLogWindow(USIGN16 CID);
IBSHLI  IBS_HLI_EnableLogEvents(USIGN16 CID, USIGN16 eventgroup);
IBSHLI  IBS_HLI_DisableLogEvents(USIGN16 CID, USIGN16 eventgroup);
IBSHLI  IBS_HLI_CloseLogWindow(USIGN16 CID);
IBSHLI  IBS_HLI_LogUserEvent(USIGN16 CID, STRINGREF UserText, HLIBOOL WithTimeStamp);
IBSHLI  IBS_HLI_LogAutoSave(USIGN16 CID, STRINGREF File, HLIBOOL Append);
#endif

/* HLI-INTERBUS-Information functions */

IBSHLI  IBS_HLI_GetControllerInfo(USIGN16 CID, T_HLI_SC_INFO REF BI);
IBSHLI  IBS_HLI_CheckBusConfiguration(USIGN16 CID, USIGN16 nDevices, T_HLI_DEVICE_DATA REF DeviceList);
IBSHLI  IBS_HLI_GetBusInfo(USIGN16 CID, T_HLI_BUS_INFO REF BI);
IBSHLI  IBS_HLI_GetDeviceInfo(USIGN16 CID, USIGN16 DeviceIndex, T_HLI_DEVICE_INFO REF DI);
IBSHLI  IBS_HLI_GetDiagnosticInfo(USIGN16 CID, T_HLI_SC_DIAG REF DI);

/* HLI-State-Machine-function */
IBSHLI  IBS_HLI_Process(USIGN16 CID);

/* HLI-process data-management */
IBSHLI  IBS_HLI_PD_Input(USIGN16 CID, VOIDREF lpAppVar);
IBSHLI  IBS_HLI_PD_Output(USIGN16 CID, VOIDREF lpAppVar);
IBSHLI  IBS_HLI_PD_In(USIGN16 CID);
IBSHLI  IBS_HLI_PD_Out(USIGN16 CID);
IBSHLI  IBS_HLI_PD_DeviceIn(USIGN16 CID, USIGN16 Segment, USIGN16 Position);
IBSHLI  IBS_HLI_PD_DeviceOut(USIGN16 CID, USIGN16 Segment, USIGN16 Position);
IBSHLI  IBS_HLI_ResetAllOutputs(USIGN16 CID);

/* for application controlled mode only */
IBSHLI  IBS_HLI_GetCyclePhase(VOID);
IBSHLI  IBS_HLI_RunCycle(VOID);
IBSHLI  IBS_HLI_RunCompleteCycle(VOID);

/* INTERBUS-PD-Object registration function */

   /* master function */
IBSHLI  IBS_HLI_RegisterPDObject(USIGN16 CID, USIGN16 PD_Type,
                                 USIGN16 segment_nr, USIGN16 segment_pos, USIGN16 PDOType,
                                 USIGN16 ByteOffset, USIGN16 BitOffset, USIGN16 Length,
                                 VOIDREF lpAppVar, HLIBOOL REF lpChangedInd );

   /* Bool objects */
IBSHLI  IBS_HLI_RegisterPDO_BOOL(USIGN16 CID, USIGN16 PD_Type,
                                 USIGN16 segment_nr, USIGN16 segment_pos,
                                 USIGN16 ByteOffset, USIGN16 BitPosition,
                                 T_IBS_BOOL REF lpAppVar, HLIBOOL REF lpChangedInd  );

   /* Bitstring objects */
IBSHLI  IBS_HLI_RegisterPDO_BITSTRING(USIGN16 CID, USIGN16 PD_Type,
                                      USIGN16 segment_nr, USIGN16 segment_pos,
                                      USIGN16 ByteOffset, USIGN16 BitPosition, USIGN16 BitLength,
                                      T_IBS_BITSTRING REF lpAppVar, HLIBOOL REF lpChangedInd  );

   /* Byte objects */
IBSHLI  IBS_HLI_RegisterPDO_BYTE(USIGN16 CID, USIGN16 PD_Type,
                                 USIGN16 segment_nr, USIGN16 segment_pos,
                                 USIGN16 ByteOffset,
                                 T_IBS_BYTE REF lpAppVar, HLIBOOL REF lpChangedInd  );

   /* WORD objects */
IBSHLI  IBS_HLI_RegisterPDO_WORD(USIGN16 CID, USIGN16 PD_Type,
                                 USIGN16 segment_nr, USIGN16 segment_pos,
                                 USIGN16 ByteOffset,
                                 T_IBS_WORD REF lpAppVar, HLIBOOL REF lpChangedInd  );

   /* DWORD objects */
IBSHLI  IBS_HLI_RegisterPDO_DWORD(USIGN16 CID, USIGN16 PD_Type,
                                  USIGN16 segment_nr, USIGN16 segment_pos,
                                  USIGN16 ByteOffset,
                                  T_IBS_DWORD REF lpAppVar, HLIBOOL REF lpChangedInd  );

   /* Unregistration function */
IBSHLI  IBS_HLI_UnregisterPDObject(USIGN16 CID, VOIDREF lpAppVar );



/* INTERBUS-PCP functions */

IBSHLI  IBS_HLI_PCP_SetConnectTimeout(USIGN16 CID, USIGN16 seconds);
IBSHLI  IBS_HLI_PCP_DeviceState(USIGN16 CID, USIGN16 Segment, USIGN16 Position);
IBSHLI  IBS_HLI_PCP_ReconnectDevice(USIGN16 CID, USIGN16 Segment, USIGN16 Position);
IBSHLI  IBS_HLI_PCP_ReadRequest(USIGN16 CID, T_HLI_PCP_RW REF lpRRQ);
IBSHLI  IBS_HLI_PCP_ReadService(USIGN16 CID, T_HLI_PCP_RW REF lpRRQ, USIGN16 timeout_ms);
IBSHLI  IBS_HLI_PCP_WriteRequest(USIGN16 CID, T_HLI_PCP_RW REF lpRRQ);
IBSHLI  IBS_HLI_PCP_WriteService(USIGN16 CID, T_HLI_PCP_RW REF lpRRQ, USIGN16 timeout_ms);
IBSHLI  IBS_HLI_PCP_IdentifyRequest(USIGN16 CID, T_HLI_PCP_DEVICE_INFO REF lpDI);
IBSHLI  IBS_HLI_PCP_IdentifyService(USIGN16 CID, T_HLI_PCP_DEVICE_INFO REF lpDI, USIGN16 timeout_ms);
IBSHLI  IBS_HLI_PCP_RegisterClientObject(USIGN16 CID, T_HLI_PCP_RW REF lpRRQ);
IBSHLI  IBS_HLI_PCP_StartRequest(USIGN16 CID, T_HLI_PCP_PRG REF lpPRG);
IBSHLI  IBS_HLI_PCP_StartService(USIGN16 CID, T_HLI_PCP_PRG REF lpPRG, USIGN16 timeout_ms);
IBSHLI  IBS_HLI_PCP_StopRequest(USIGN16 CID, T_HLI_PCP_PRG REF lpPRG);
IBSHLI  IBS_HLI_PCP_StopService(USIGN16 CID, T_HLI_PCP_PRG REF lpPRG, USIGN16 timeout_ms);
IBSHLI  IBS_HLI_PCP_ResumeRequest(USIGN16 CID, T_HLI_PCP_PRG REF lpPRG);
IBSHLI  IBS_HLI_PCP_ResumeService(USIGN16 CID, T_HLI_PCP_PRG REF lpPRG, USIGN16 timeout_ms);
IBSHLI  IBS_HLI_PCP_ResetRequest(USIGN16 CID, T_HLI_PCP_PRG REF lpPRG);
IBSHLI  IBS_HLI_PCP_ResetService(USIGN16 CID, T_HLI_PCP_PRG REF lpPRG, USIGN16 timeout_ms);

#ifdef HLI_PCP_RW_NAME_SUPPORT
/* Read and Write with Name functions */ 
IBSHLI   IBS_HLI_PCP_WriteWithNameService(USIGN16 CID, T_HLI_PCP_RW_NAME REF lpWRQ, USIGN16 timeout_ms); 
IBSHLI   IBS_HLI_PCP_WriteWithNameRequest(USIGN16 CID, T_HLI_PCP_RW_NAME REF lpWRQ); 
IBSHLI   IBS_HLI_PCP_ReadWithNameService(USIGN16 CID, T_HLI_PCP_RW_NAME REF lpWRQ, USIGN16 timeout_ms); 
IBSHLI   IBS_HLI_PCP_ReadWithNameRequest(USIGN16 CID, T_HLI_PCP_RW_NAME REF lpWRQ); 
#endif


/* INTERBUS-PCP-object translation functions */

#ifdef HLI_PCP_FILE_SUPPORT

IBSHLI  IBS_HLI_PCP_LoadObjectByName(STRINGREF Filename, STRINGREF ObjectName,
                                     T_HLI_PCP_RW REF lpPCPObj);
IBSHLI  IBS_HLI_PCP_LoadObjectByIndex(STRINGREF Filename, USIGN16 Index,
                                      T_HLI_PCP_RW REF lpPCPObj);
IBSHLI  IBS_HLI_PCP_LoadObjectByEntry(STRINGREF Filename, USIGN16 Entry,
                                      T_HLI_PCP_RW REF lpPCPObj);
#endif

IBSHLI  IBS_HLI_PCP_InitObject(T_HLI_PCP_RW REF lpPCPObj, USIGN16 Segment, USIGN16 Position,
                               USIGN16 Index, USIGN16 Subindex, USIGN16 Type, USIGN16 nElements,
                               VOIDREF lpData);
IBSHLI  IBS_HLI_PCPObject_SetInt16(T_HLI_PCP_RW REF lpPCPObj, USIGN16 Offset, USIGN16 VInt16);
IBSHLI  IBS_HLI_PCPObject_SetInt32(T_HLI_PCP_RW REF lpPCPObj, USIGN16 Offset, USIGN32 VInt32);
IBSHLI  IBS_HLI_PCPObject_SetFloat(T_HLI_PCP_RW REF lpPCPObj, USIGN16 Offset, FLOAT VFloat);
IBSHLI  IBS_HLI_PCPObject_SetString(T_HLI_PCP_RW REF lpPCPObj, USIGN16 Offset, STRINGREF lpS);

IBSHLI  IBS_HLI_PCPObject_GetInt16(T_HLI_PCP_RW REF lpPCPObj, USIGN16 Offset, USIGN16 REF lpInt16);
IBSHLI  IBS_HLI_PCPObject_GetInt32(T_HLI_PCP_RW REF lpPCPObj, USIGN16 Offset, USIGN32 REF lpInt32);
IBSHLI  IBS_HLI_PCPObject_GetFloat(T_HLI_PCP_RW REF lpPCPObj, USIGN16 Offset, FLOAT REF lpFloat);
IBSHLI  IBS_HLI_PCPObject_GetString(T_HLI_PCP_RW REF lpPCPObj, USIGN16 Offset, STRINGREF lpS);

/* HLI-INTERBUS-Controller-Services */
IBSHLI  IBS_HLI_ActivateWatchdog(USIGN16 CID, USIGN16 WD_Timeout);
IBSHLI  IBS_HLI_ActivateWatchdogEx(USIGN16 CID, USIGN16 WD_Timeout);

/* INTERBUS services */

IBSHLI  IBS_HLI_StartBus(USIGN16 CID);
IBSHLI  IBS_HLI_StopBus(USIGN16 CID);
IBSHLI  IBS_HLI_AlarmStop(USIGN16 CID);
IBSHLI  IBS_HLI_SwitchRemoteBus(USIGN16 CID, USIGN16 segment_nr, USIGN16 switch_code);
IBSHLI  IBS_HLI_SwitchLocalBus(USIGN16 CID, USIGN16 segment_nr, USIGN16 switch_code);
IBSHLI  IBS_HLI_SwitchGroup(USIGN16 CID, USIGN16 group, USIGN16 alternative, USIGN16 switch_code);
IBSHLI  IBS_HLI_ChangeDeviceState(USIGN16 CID, USIGN16 Segment, USIGN16 Position, USIGN16 new_state );


#ifdef HLI_NODE_2

/* COP486 : HOST communication functions */

IBSHLI  IBS_HLI_SendMessageToHost(T_HLI_MSG REF msg, USIGN16 timeout);
IBSHLI  IBS_HLI_ReceiveMessageFromHost(T_HLI_MSG REF msg);
#ifdef HLI_OS_VXWORKS
IBSHLI  IBS_HLI_WaitForMessageFromHost(T_HLI_MSG REF msg, int timeout);
#endif
IBSHLI  IBS_HLI_ReadDataFromHost(USIGN16 from, USIGN16 nbytes, USIGN8 * dest);
IBSHLI  IBS_HLI_WriteDataToHost(USIGN16 to, USIGN16 nbytes, USIGN8 * source);

#endif

#ifdef HLI_COPCOM_SUPPORT

/* HOST : COP communication functions */

IBSHLI  IBS_HLI_SendMessageToCop(USIGN16 CID, T_HLI_MSG REF msg, USIGN16 timeout);
IBSHLI  IBS_HLI_ReceiveMessageFromCop(USIGN16 CID, T_HLI_MSG REF msg);
IBSHLI  IBS_HLI_WaitForMessageFromCop(USIGN16 CID, T_HLI_MSG REF msg, USIGN32 timeout);

IBSHLI  IBS_HLI_ReadDataFromCop(USIGN16 CID, USIGN16 from, USIGN16 nbytes, USIGN8 * dest);
IBSHLI  IBS_HLI_WriteDataToCop(USIGN16 CID, USIGN16 to, USIGN16 nbytes, USIGN8 * source);
IBSHLI  IBS_HLI_GetCopDiagnostic(USIGN16 CID, USIGN16 REF StateReg, USIGN16 REF ParameterReg);


#endif


#ifdef __cplusplus
 }                       /* End of extern "C" { */
#endif  /* __cplusplus */


#endif  /* IBS_G4_HLI_H */


/*--------------------------------------------------------------------*/
/*                   END OF FILE                                      */
/*--------------------------------------------------------------------*/


