/**************************************************************************/
/*********     INTERFAZ BASICA INTERBUS IBS PCI SC-IT  ********************/
/**************************************************************************/
/********* Autor:     ANTONIO ROMEO TELLO              ********************/
/********* Paquete:   InterBus_E_S                     ********************/
/********* Lenguaje:  ansi C                           ********************/
/********* Mision:    Proporciona herramientas para    ********************/
/*********            realizar la entradas/salida      ********************/
/*********            Analgica               .        ********************/
/*********            HLI-Export-Filter Version 2.00   ********************/
/********* Fecha:     ABRIL- 2005                      ********************/
/**************************************************************************/

/* ------ include standard defs --------- */
#include <stdio.h>

/* ------ include HLI-Header --------- */

#include "g4hliw32.h"

#ifdef __cplusplus
extern "C" {            /* Assume C declarations for C++ */
#endif  /* __cplusplus */


/* ------ macros --------------------- */

#ifndef HLI_VERIFY
 #define HLI_VERIFY(x);        if( (ret = x) != HLI_OKAY) return ret;
#endif

/*
+-------------------------------------------------------------------+
|                         GLOBAL VARIABLES                          |
+-------------------------------------------------------------------+*/

/* ------ Global HLI-State object for this controller --------- */

 T_HLI_STATE         PCI1;

/* ------ Bus configuration list ----- */

 T_HLI_DEVICE_DATA   PCI1_DeviceList[3] =
                     {
                     	{ 1, 0,   4, 0, 0, 255, 255, 0x0, "" } ,
                     	{ 1, 1, 127, 2, 1, 255, 255, 0x0, "" } ,
                     	{ 1, 2,  91, 2, 1, 255, 255, 0x0, "" }
                     };

/* ------ Global process data objects ------------------------ */

 T_IBS_WORD          AI1;
 T_IBS_WORD          AI2;
 T_IBS_WORD          CONTROLWORD1;
 T_IBS_WORD          CONTROLWORD2;
 T_IBS_WORD          AO1;
 T_IBS_WORD          AO2;
/*
+-------------------------------------------------------------------+
|                         PUBLIC FUNCTIONS                          |
+-------------------------------------------------------------------+*/


/*
#####################################################################
#                                                                   #
#      Call this function for startup of the INTERBUS system        #
#                                                                   #
#####################################################################
*/
HLIRET init_IB(void)
{
	HLIRET ret;

	/* Call the HLI Initialization function */
	ret = IBS_HLI_Init_CFG(PCISC1, &PCI1, IBS_STANDARD, 3, PCI1_DeviceList);
	if( ret == HLI_OKAY )
	{
		/* --- Process data object registration --- */

		IBS_HLI_RegisterPDObject(PCISC1, IBS_PDO_INPUT,  1, 1, IBS_PDO_WORD, 0, 0, 16, &AI1, NULL);
		IBS_HLI_RegisterPDObject(PCISC1, IBS_PDO_INPUT,  1, 1, IBS_PDO_WORD, 2, 0, 16, &AI2, NULL);
		IBS_HLI_RegisterPDObject(PCISC1, IBS_PDO_OUTPUT, 1, 1, IBS_PDO_WORD, 0, 0, 16, &CONTROLWORD1, NULL);
		IBS_HLI_RegisterPDObject(PCISC1, IBS_PDO_OUTPUT, 1, 1, IBS_PDO_WORD, 2, 0, 16, &CONTROLWORD2, NULL);
		IBS_HLI_RegisterPDObject(PCISC1, IBS_PDO_OUTPUT, 1, 2, IBS_PDO_WORD, 0, 0, 16, &AO1, NULL);
		IBS_HLI_RegisterPDObject(PCISC1, IBS_PDO_OUTPUT, 1, 2, IBS_PDO_WORD, 2, 0, 16, &AO2, NULL);
		
		/* --- Bus operation start --- */
		
		IBS_HLI_StartBus(PCISC1);
	}
	return ret;
}


/*
#####################################################################
#                                                                   #
#                  Shut-down of the INTERBUS system -               #
#     Call this function before your application terminates !       #
#                                                                   #
#####################################################################
*/

void release_IB(void)
{
	IBS_HLI_Exit(PCISC1);
}


/*
#####################################################################
#                                                                   #
#             Analog input                                          #
#                                                                   #
#####################################################################
*/

double analog_input_IB (int analog_channel)
{
	short int resultado;
	if (analog_channel == 1)
	{
		CONTROLWORD1=0x8001;
		IBS_HLI_PD_Output(PCISC1, &CONTROLWORD1);
		IBS_HLI_PD_Input(PCISC1, &AI1);
		resultado = (short int)(AI1);
	}
	else
	{
		CONTROLWORD2=0x8001;
		IBS_HLI_PD_Output(PCISC1, &CONTROLWORD2);
		IBS_HLI_PD_Input(PCISC1, &AI2);
		resultado = (short int)(AI2);
	}
	return (((double)(resultado))/3000.0);
}

/*
#####################################################################
#                                                                   #
#             Analog output                                         #
#                                                                   #
#####################################################################
*/

void analog_output_IB (double data)
{
	if (data < -10) data=-10;
	else if (data > 10) data=10;
	data=data*3000.0;
	AO1 = (short int)(data);	// Two's complement
    IBS_HLI_PD_Output(PCISC1, &AO1);
}


#ifdef __cplusplus
} /* extern C*/
#endif

/* ----------------------------- END OF FILE ------------------------------ */
